#include "main.h"
#include <Arduino.h>
#include "epaperdisplay.h"
#include "button.h"
#include "coinacceptor.h"
#include "lnurlutil.h"
#include "config.h"

EpaperDisplay epDisp(DSPLY_PIN_CS, DSPLY_PIN_DC, DSPLY_PIN_RST, DSPLY_PIN_BUSY);
AtmButton button(BUTTON_PIN, LED_BUTTON_PIN);
CoinAcceptor cacc(COIN_PIN, MOSFET_PIN, COINS, COINS_COUNT, COIN_PULSE_WIDTH_MS);

static State current_state = INIT;
static State next_state = INIT;

std::string lnurl_str = "";

void init_atm();

void setup()
{
  Serial.begin(115200);
  epDisp.init();        // Init eInk display

  epDisp.initScreen();

  cacc.begin();         // Init Coin acceptor
  cacc.disable();       // Disable coin detection
  button.begin();       // Init button
  button.wasPressed();  // Reset button - if already pressed

  while( true == button.iswaitingUntilPressed(/* maxTimeToWaitMs0= */ 0, /* blinkPeriodMs= */ 1000))
  {}

}

void loop()
{
  switch (current_state) {
    case INIT:
        init_atm();
        button.wasPressed();
        next_state = WAIT_FOR_COIN;
        break;
    case WAIT_FOR_COIN:
        if(button.wasPressed())
        {
          epDisp.showInsertedAmount("10,00");
          button.wasPressed();
          next_state = PROCESS_COIN;
        }
        break;
    case PROCESS_COIN:
        //update coin ammount until OK or CANCEL button is pressed
        if(button.wasPressed())
        {        
          next_state = GENERATE_QR;
          button.wasPressed();
        }
        break;

    case GENERATE_QR:
        lnurl_str = lnurlutil::createQrContent(0.05);
        epDisp.qrCode(lnurl_str.c_str());
        next_state = PAYMENT_CONFIRMED;
        button.wasPressed();        
        break;
    case PAYMENT_CONFIRMED:
        if(button.wasPressed())
        {
          next_state = INIT;
          button.wasPressed();
        }
        break;

    case ERROR_STATE:
        next_state = WAIT_FOR_COIN;
        break;
  }

  current_state = next_state;
}

void init_atm()
{
  button.ledOff();
  epDisp.homeScreen();
  cacc.enable();
}

